﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Windows.Data;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GanttView;
using Telerik.Windows.Controls.Scheduling;

namespace GridSorting
{
    public class TasksDataSource : ViewModelBase
    {
        public TasksDataSource()
        {
            var start = new DateTime(2012, 9, 10);
            this._VisibleRange = new VisibleRange(start, start.AddMonths(1));
            var tasksSource = new ObservableCollection<CustomGanttTask>
			{
				new CustomGanttTask { Start = start.AddDays(10), End = start.AddDays(13), Title = "Task 1", MyProperty = "MyProperty 1" },
				new CustomGanttTask { Start = start.AddDays(5), End = start.AddDays(10), Title = "Task 2" , MyProperty = "MyProperty 2"},
				new CustomGanttTask { Start = start.AddDays(12), End = start.AddDays(25), Title = "Task 3", MyProperty = "MyProperty 3", 
					Children = {
						new CustomGanttTask { Start = start.AddDays(12), End = start.AddDays(14), Title = "Task 3.1" , MyProperty = "MyProperty 4"},
						new CustomGanttTask { Start = start.AddDays(20), End = start.AddDays(25), Title = "Task 3.2" , MyProperty = "MyProperty 5"},
						new CustomGanttTask { Start = start.AddDays(13), End = start.AddDays(19), Title = "Task 3.3" , MyProperty = "MyProperty 6"},
					}},
				new CustomGanttTask { Start = start.AddDays(20), End = start.AddMonths(1).AddDays(-3), Title = "Task 4" , MyProperty = "MyProperty 7"},
				new CustomGanttTask { Start = start, End = start.AddMonths(1), Title = "A lot of subtasks", MyProperty = "MyProperty 8" }
			};
            this._TasksSource = tasksSource;
        }

        private IDateRange _VisibleRange;

        /// <Summary>Gets or sets VisibleRange and notfinies for changes</Summary>
        public IDateRange VisibleRange
        {
            get { return this._VisibleRange; }
            set
            {
                if (this._VisibleRange != value)
                {
                    this._VisibleRange = value;
                    this.OnPropertyChanged(() => this.VisibleRange);
                }
            }
        }

        private IEnumerable _TasksSource;

        /// <Summary>Gets or sets TasksSource and notfinies for changes</Summary>
        public IEnumerable TasksSource
        {
            get { return this._TasksSource; }
            set
            {
                if (this._TasksSource != value)
                {
                    this._TasksSource = value;
                    this.OnPropertyChanged(() => this.TasksSource);
                }
            }
        }

        public void SourtBy(Binding propertyBinding, bool isAscending)
        {
            var propertyName = propertyBinding.Path.Path.Split('.').First();

            var tasks = this._TasksSource as IList<CustomGanttTask>;
            this.TasksSource = null;

            if (propertyName == "Title")
            {
                Sort(tasks, t => t.Title, isAscending);
            }
            else if (propertyName == "Start")
            {
                Sort(tasks, t => t.Start, isAscending);
            }
            else if (propertyName == "End")
            {
                Sort(tasks, t => t.End, isAscending);
            }
            else if (propertyName == "MyProperty")
            {
                Sort(tasks, t => t.MyProperty, isAscending);
            }

            this.TasksSource = tasks;
        }

        private void Sort<T>(IList<CustomGanttTask> tasks, Func<CustomGanttTask, T> sortKeySelector, bool isAscending)
        {
            var sortedTasks = (isAscending ? tasks.OrderBy(sortKeySelector) : tasks.OrderByDescending(sortKeySelector)).ToArray();
            tasks.Clear();
            foreach (var task in sortedTasks)
            {
                tasks.Add(task);
            }
        }
    }
}